package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import co.hyperverge.hypersnapsdk.activities.HVDocsActivity;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVDocConfig;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;

import static co.hyperverge.hypersnapsdk.hypersnapsdk_flutter.FLHVNetworkHelper.convertJsonToMap;
import static co.hyperverge.hypersnapsdk.hypersnapsdk_flutter.FLHVNetworkHelper.convertMapToJson;

public class FLHVDocsCapture {
    public String docType;
    public Float aspectRatio;
    HVDocConfig docConfig = null;
    HVDocConfig.Document doc;
    boolean hasBeenCalled;

    public HVDocConfig getDocConfig() {
        if (this.docConfig == null) {
            this.docConfig = new HVDocConfig();

        }
        return this.docConfig;

    }

    public void setOCRAPIDetails(String ocrEndpoint, String documentSideString, Map<String, Object> params,
                                 Map<String, Object> headers) throws JSONException {
        String[] documentSideParams = documentSideString.split("\\.");
        HVDocConfig.DocumentSide documentSide = getDocumentSide(documentSideParams[2], HVDocConfig.DocumentSide.class);

        JSONObject paramsObj = new JSONObject();
        if (params != null) {
            paramsObj = convertMapToJson(params);
        }

        JSONObject headersObj = new JSONObject();
        if (headers != null) {
            headersObj = convertMapToJson(headers);
        }

        if (documentSide != null) {
            getDocConfig().setOCRDetails(ocrEndpoint,
                    documentSide,
                    paramsObj.toString(),
                    headersObj.toString());
        }
    }

    private <T> HVDocConfig.DocumentSide getDocumentSide(String documentSide, Class<T> type) {
        switch (documentSide) {
            case "FRONT":
            case "front":
                return HVDocConfig.DocumentSide.FRONT;
            case "BACK":
            case "back":
                return HVDocConfig.DocumentSide.BACK;
            default: {
                return null;
            }
        }
    }

    public void setCustomUIStrings(String customStrings) {
        try {
            JSONObject stringObj = new JSONObject();
            if (customStrings == null && !customStrings.trim().isEmpty())
                stringObj = new JSONObject(customStrings);
            getDocConfig().setCustomUIStrings(stringObj);
        } catch (JSONException e) {
            Log.e("docsCapture", e.getMessage());
        }
    }


    public void setAspectRatio(Float aspectRatioValue) {
        aspectRatio = aspectRatioValue;
    }


    public void setShouldAddPadding(Boolean shouldSetPadding) {
        getDocConfig().setShouldAddPadding(shouldSetPadding.booleanValue());

    }


    public void setPadding(Number padding) {
        getDocConfig().setPadding((float) padding);

    }


    public void setShouldShowFlashIcon(Boolean shouldShowFlashIcon) {
        getDocConfig().setShouldShowFlashIcon(shouldShowFlashIcon.booleanValue());
    }


    public void setShouldShowReviewScreen(Boolean shouldShowReviewScreen) {
        getDocConfig().setShouldShowReviewScreen(shouldShowReviewScreen.booleanValue());
    }


    public void setDocCaptureSubText(String subText) {
        getDocConfig().setDocCaptureSubText(subText);
    }


    public void setDocCaptureDescription(String description) {
        getDocConfig().setDocCaptureDescription(description);
    }


    public void setShouldShowInstructionPage(Boolean shouldShowInstructionPage) {
        getDocConfig().setShouldShowInstructionPage(shouldShowInstructionPage.booleanValue());
    }


    public HVDocConfig getConfig() {
        return getDocConfig();
    }


    public void setDocumentType(String docTypeValue) {
        try {
            this.docType = docTypeValue.split("\\.")[1];
            if (this.docType == null) {
                this.docType = "Card";
            }

            this.doc = HVDocConfig.Document.valueOf(docType);
            if (aspectRatio != null)
                this.doc.setAspectRatio(this.aspectRatio);

            getDocConfig().setDocumentType(this.doc);
        } catch (Exception exp) {
            Log.e("docsCapture", exp.getMessage());
        }
    }


    public void setDocCaptureTitle(String titleText) {
        getDocConfig().setDocCaptureTitle(titleText);
    }


    public void setDocReviewTitle(String docReviewTitle) {

        getDocConfig().setDocReviewTitle(docReviewTitle);
    }


    public void setDocReviewDescription(String docReviewDescription) {
        getDocConfig().setDocReviewDescription(docReviewDescription);

    }


    public void start(Context context, final DocCaptureCompletionHandler callback) {
        hasBeenCalled = false;


        HVDocsActivity.start(context, getDocConfig(), new DocCaptureCompletionHandler() {
            @Override
            public void onResult(HVError error, HVResponse hvResponse) {
                callback.onResult(error, hvResponse);
            }
        });
    }

    public ArrayList<Map<String, Object>> getMapsFromOnResult(HVError error, HVResponse hvResponse) {
        ArrayList<Map<String, Object>> arrayList = new ArrayList<>();
        Map<String, Object> errorObj = new HashMap<>();
        Map<String, Object> resultObj = new HashMap<>();

        try {

            if (error != null) {
                errorObj.put("errorCode", error.getErrorCode());
                errorObj.put("errorMessage", error.getErrorMessage());
                if (!hasBeenCalled) {
                    hasBeenCalled = true;
                }
            } else {
                JSONObject result = hvResponse.getApiResult();
                JSONObject headers = hvResponse.getApiHeaders();

                String imageURI = hvResponse.getImageURI();
                String fullImageUri = hvResponse.getFullImageURI();
                String action = hvResponse.getAction();
                String retakeMessage = hvResponse.getRetakeMessage();

                if (result != null) {

                    resultObj = convertJsonToMap(result);
                    resultObj.put("apiResult", result.toString());
                    resultObj.put("imageUri", imageURI);
                    if (fullImageUri != null && !fullImageUri.isEmpty()) {
                        resultObj.put("fullImageUri", fullImageUri);
                    }
                    if (retakeMessage != null && !retakeMessage.isEmpty()) {
                        resultObj.put("retakeMessage", retakeMessage);
                    }
                    if (action != null && !action.isEmpty()) {
                        resultObj.put("action", action);
                    }
                }
                if (headers != null) {
                    try {
                        resultObj.put("apiHeaders", headers.toString());
                    } catch (Exception e) {
                        Log.e("docsCapture", (e.getMessage()));
                    }
                }
                if (!hasBeenCalled) {
                    hasBeenCalled = true;
                }
            }
        } catch (Exception e) {
            Log.e("docsCapture", (e.getMessage()));
        }
        arrayList.add(errorObj);
        arrayList.add(resultObj);
        return arrayList;
    }


}
