package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;

import android.content.Context;
import android.os.Build;
import android.util.Log;

import androidx.annotation.RequiresApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Objects;

import co.hyperverge.hypersnapsdk.activities.HVFaceActivity;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVFaceConfig;
import co.hyperverge.hypersnapsdk.objects.HVResponse;

public class FLHVFaceCapture {
    public String liveness;
    HVFaceConfig faceConfig;
    boolean hasBeenCalled;

    public String getName() {
        return "RNHVFaceCapture";
    }


    public void setLivenessEndpoint(String livenessEndpoint) {
        getFaceConfig().setLivenessEndpoint(livenessEndpoint);
    }

    public HVFaceConfig getFaceConfig() {
        if (this.faceConfig == null) {
            this.faceConfig = new HVFaceConfig();
        }
        return this.faceConfig;

    }


    public void setShouldReturnFullImageUrl(Boolean shouldReturnFullImageUrl) {
        getFaceConfig().setShouldReturnFullImageUrl(shouldReturnFullImageUrl.booleanValue());
    }


    public void setClientID(String clientID) {
        getFaceConfig().setClientID(clientID);

    }


    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setCustomUIStrings(String customStrings) {
        try {
            JSONObject stringObj = new JSONObject();
            if (customStrings == null && !customStrings.trim().isEmpty())
                stringObj = new JSONObject(customStrings);
            getFaceConfig().setCustomUIStrings(stringObj);
        } catch (JSONException e) {
            Log.e(getName(), Objects.requireNonNull(e.getMessage()));
        }
    }


    public void setShouldShowCameraSwitchButton(Boolean shouldShow) {
        getFaceConfig().setShouldShowCameraSwitchButton(shouldShow.booleanValue());
    }



    public void setShouldShowInstructionPage(Boolean shouldShowInstructionPage) {
        getFaceConfig().setShouldShowInstructionPage(shouldShowInstructionPage.booleanValue());
    }



    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setLivenessMode(String livenessValue) {
        try {
            liveness = livenessValue.split("\\.")[1];
            HVFaceConfig.LivenessMode livenessMode = HVFaceConfig.LivenessMode.valueOf(liveness);
            getFaceConfig().setLivenessMode(livenessMode);
        } catch (Exception exp) {
            Log.e(getName(), Objects.requireNonNull(exp.getMessage()));
        }
    }


    public void setFaceCaptureTitle(String faceCaptureTitle) {
        getFaceConfig().setFaceCaptureTitle(faceCaptureTitle);
    }


    public void setShouldUseBackCamera(Boolean shouldUseBackCamera) {
        getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera.booleanValue());
    }


    public void setShouldEnableDataLogging(Boolean dataLogging) {
        getFaceConfig().setShouldEnableDataLogging(dataLogging.booleanValue());
    }


    public void setShouldAddPadding(Boolean shouldSetPadding) {
        getFaceConfig().setShouldEnablePadding(shouldSetPadding.booleanValue());

    }


    public void setPadding(Number leftPadding, Number rightPadding, Number topPadding, Number bottomPadding) {
        getFaceConfig().setPadding((float) (leftPadding), (float) rightPadding, (float) topPadding, (float) bottomPadding);
    }


    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setLivenessAPIParameters(String params) {
        try {
            getFaceConfig().setLivenessAPIParameters(new JSONObject(params));
        } catch (JSONException e) {
            Log.e(getName(), Objects.requireNonNull(e.getMessage()));
        }
    }


    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void setLivenessAPIHeaders(String headers) {
        try {
            getFaceConfig().setLivenessAPIHeaders(new JSONObject(headers));
        } catch (JSONException e) {
            Log.e(getName(), Objects.requireNonNull(e.getMessage()));
        }
    }



    public void start(Context context, final FaceCaptureCompletionHandler callback) {
        hasBeenCalled = false;

        HVFaceActivity.start(context, getFaceConfig(), new FaceCaptureCompletionHandler() {
            @RequiresApi(api = Build.VERSION_CODES.KITKAT)
            @Override
            public void onResult(HVError error, HVResponse hvResponse) {
                callback.onResult(error,hvResponse);
            }
        });

    }

}
