package co.hyperverge.hypersnapsdk.hypersnapsdk_flutter;


import android.app.Activity;
import android.content.Context;
import android.os.Build;
import android.util.Log;

import androidx.annotation.NonNull;
import androidx.annotation.RequiresApi;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;


import co.hyperverge.hypersnapsdk.listeners.APICompletionCallback;
import co.hyperverge.hypersnapsdk.listeners.DocCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.FaceCaptureCompletionHandler;
import co.hyperverge.hypersnapsdk.listeners.QRScannerCompletionHandler;
import co.hyperverge.hypersnapsdk.objects.HVError;
import co.hyperverge.hypersnapsdk.objects.HVResponse;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.embedding.engine.plugins.activity.ActivityAware;
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
/** HypersnapsdkFlutterPlugin */
public class HypersnapsdkFlutterPlugin implements FlutterPlugin, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private MethodChannel hyperSnapSDKChannel, hyperSnapParamsChannel, hvDocsCaptureChannel, hvFaceCaptureChannel, hvNetworkHelperChannel, hvQRScanCaptureChannel;


  Context context;
  Activity activity;

  FLHyperSnapSDK flHypersnapSDK = new FLHyperSnapSDK();
  FLHyperSnapParams flHyperSnapParams = new FLHyperSnapParams();
  FLHVDocsCapture flhvDocsCapture = new FLHVDocsCapture();
  FLHVFaceCapture flhvFaceCapture = new FLHVFaceCapture();
  FLHVQRScanCapture flhvqrScanCapture = new FLHVQRScanCapture();
  FLHVNetworkHelper flhvNetworkHelper = new FLHVNetworkHelper();

  MethodCallHandler hyperSnapSDKMethodCallHandler = new MethodCallHandler() {
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
      if (call.method.equalsIgnoreCase("initialize")) {
        String appID = call.argument("appID"),
                appKey = call.argument("appKey"),
                region = call.argument("region");
        flHypersnapSDK.initialize(activity, appID, appKey, region);
        result.success("Initialised");
      } else if (call.method.equalsIgnoreCase("startUserSession")) {
        String userID = call.argument("userID");
        flHypersnapSDK.startUserSession(userID);
      } else if (call.method.equalsIgnoreCase("endUserSession")) {
        flHypersnapSDK.endUserSession();
      } else {
        result.notImplemented();
      }
    }
  };

  MethodCallHandler hyperSnapParamsMethodCallHandler = new MethodCallHandler() {
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
      if (call.method.equalsIgnoreCase("getValidParams")) {
        result.success(flHyperSnapParams.getConstants());
      } else {
        result.notImplemented();
      }
    }
  };

  MethodCallHandler hvDocsCaptureMethodCallHandler = new MethodCallHandler() {
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
      if (call.method.equalsIgnoreCase("docsCaptureSetOCRAPIDetails")) {
        String ocrEndpoint = call.argument("ocrEndpoint");
        String documentSideString = call.argument("documentSideString");
        Map<String, Object> params = call.argument("params");
        Map<String, Object> headers = call.argument("headers");

        try {
          flhvDocsCapture.setOCRAPIDetails(ocrEndpoint, documentSideString, params, headers);
        } catch (JSONException e) {
          e.printStackTrace();
        }
      } else if (call.method.equalsIgnoreCase("docsCaptureSetCustomUIStrings")) {
        String customStrings = call.argument("customStrings");
        flhvDocsCapture.setCustomUIStrings(customStrings);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetAspectRatio")) {
        double aspectRatioValue = call.argument("aspectRatioValue");
        flhvDocsCapture.setAspectRatio((float) aspectRatioValue);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetShouldAddPadding")) {
        boolean shouldSetPadding = call.argument("shouldSetPadding");
        flhvDocsCapture.setShouldAddPadding(shouldSetPadding);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetPadding")) {
        double padding = call.argument("padding");
        flhvDocsCapture.setPadding(padding);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetShouldShowFlashIcon")) {
        boolean shouldShowFlashIcon = call.argument("shouldShowFlashIcon");
        flhvDocsCapture.setShouldShowFlashIcon(shouldShowFlashIcon);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetShouldShowReviewScreen")) {
        boolean shouldShowReviewScreen = call.argument("shouldShowReviewScreen");
        flhvDocsCapture.setShouldShowReviewScreen(shouldShowReviewScreen);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocCaptureSubText")) {
        String subText = call.argument("subText");
        flhvDocsCapture.setDocCaptureSubText(subText);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocCaptureDescription")) {
        String description = call.argument("description");
        flhvDocsCapture.setDocCaptureDescription(description);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetShouldShowInstructionPage")) {
        boolean shouldShowInstructionPage = call.argument("shouldShowInstructionPage");
        flhvDocsCapture.setShouldShowInstructionPage(shouldShowInstructionPage);
      } else if (call.method.equalsIgnoreCase("docsCaptureGetConfig")) {
//                result.success(flhvDocsCapture.getConfig());
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocumentType")) {
        String docTypeValue = call.argument("docTypeValue");
        flhvDocsCapture.setDocumentType(docTypeValue);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocCaptureTitle")) {
        String titleText = call.argument("titleText");
        flhvDocsCapture.setDocCaptureTitle(titleText);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocReviewTitle")) {
        String docReviewTitle = call.argument("docReviewTitle");
        flhvDocsCapture.setDocReviewTitle(docReviewTitle);
      } else if (call.method.equalsIgnoreCase("docsCaptureSetDocReviewDescription")) {
        String docReviewDescription = call.argument("docReviewDescription");
        flhvDocsCapture.setDocReviewDescription(docReviewDescription);
      } else if (call.method.equalsIgnoreCase("docsCaptureStart")) {
        DocCaptureCompletionHandler callback = new DocCaptureCompletionHandler() {
          @Override
          public void onResult(HVError hvError, HVResponse hvResponse) {
            ArrayList<Map<String, Object>> arr = flhvDocsCapture.getMapsFromOnResult(hvError, hvResponse);
            Map<String, Object> map = new HashMap<>();
            map.put("errorObj", arr.get(0));
            map.put("resultObj", arr.get(1));
            result.success(map);
          }
        };
        flhvDocsCapture.start(activity, callback);
      } else {
        result.notImplemented();
      }
    }
  };
  MethodCallHandler hvFaceCaptureMethodCallHandler = new MethodCallHandler() {
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
      if (call.method.equalsIgnoreCase("")) {
        // TODO: Remove this if
      } else if (call.method.equalsIgnoreCase("faceCaptureStart")) {
        FaceCaptureCompletionHandler callback = new FaceCaptureCompletionHandler() {
          @Override
          public void onResult(HVError hvError, HVResponse hvResponse) {
            ArrayList<Map<String, Object>> arr = flhvDocsCapture.getMapsFromOnResult(hvError, hvResponse);
            Map<String, Object> map = new HashMap<>();
            map.put("errorObj", arr.get(0));
            map.put("resultObj", arr.get(1));
            result.success(map);
          }
        };
        flhvFaceCapture.start(activity, callback);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetLivenessEndpoint")) {
        String livenessEndpoint = call.argument("livenessEndpoint");
        flhvFaceCapture.setLivenessEndpoint(livenessEndpoint);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldReturnFullImageUrl")) {
        boolean shouldReturnFullImageUrl = call.argument("shouldReturnFullImageUrl");
        flhvFaceCapture.setShouldReturnFullImageUrl(shouldReturnFullImageUrl);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetClientID")) {
        String clientID = call.argument("clientID");
        flhvFaceCapture.setClientID(clientID);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetCustomUIStrings")) {
        String customStrings = call.argument("customStrings");
        flhvFaceCapture.setCustomUIStrings(customStrings);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldShowCameraSwitchButton")) {
        boolean shouldShow = call.argument("shouldShow");
        flhvFaceCapture.setShouldShowCameraSwitchButton(shouldShow);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldShowInstructionPage")) {
        boolean shouldShowInstructionPage = call.argument("shouldShowInstructionPage");
        flhvFaceCapture.setShouldShowInstructionPage(shouldShowInstructionPage);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetLivenessMode")) {
        String livenessValue = call.argument("livenessValue");
        flhvFaceCapture.setLivenessMode(livenessValue);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetFaceCaptureTitle")) {
        String faceCaptureTitle = call.argument("faceCaptureTitle");
        flhvFaceCapture.setFaceCaptureTitle(faceCaptureTitle);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldUseBackCamera")) {
        boolean shouldUseBackCamera = call.argument("shouldUseBackCamera");
        flhvFaceCapture.setShouldUseBackCamera(shouldUseBackCamera);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldEnableDataLogging")) {
        boolean dataLogging = call.argument("dataLogging");
        flhvFaceCapture.setShouldEnableDataLogging(dataLogging);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetShouldAddPadding")) {
        boolean shouldSetPadding = call.argument("shouldSetPadding");
        flhvFaceCapture.setShouldAddPadding(shouldSetPadding);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetPadding")) {
        double leftPadding = call.argument("leftPadding");
        double rightPadding = call.argument("rightPadding");
        double topPadding = call.argument("topPadding");
        double bottomPadding = call.argument("bottomPadding");
        flhvFaceCapture.setPadding(leftPadding, rightPadding, topPadding, bottomPadding);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetLivenessAPIParameters")) {
        String params = call.argument("params");
        flhvFaceCapture.setLivenessAPIParameters(params);
      } else if (call.method.equalsIgnoreCase("faceCaptureSetLivenessAPIHeaders")) {
        String headers = call.argument("headers");
        flhvFaceCapture.setLivenessAPIHeaders(headers);
      } else {
        result.notImplemented();
      }
    }
  };
  MethodCallHandler hvNetworkHelperMethodCallHandler = new MethodCallHandler() {
    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
      if (call.method.equalsIgnoreCase("networkHelperMakeOCRCall")) {
        APICompletionCallback callback = new APICompletionCallback() {
          @Override
          public void onResult(HVError hvError, HVResponse hvResponse) {
            ArrayList<Map<String, Object>> arr = flhvNetworkHelper.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);

            Map<String, Object> map = new HashMap<>();
            map.put("errorObj", arr.get(0));
            map.put("resultObj", arr.get(1));
            result.success(map);
          }
        };
        String endpoint = call.argument("endpoint");
        String documentUri = call.argument("documentUri");
        Map<String, Object> parameters = call.argument("parameters");
        Map<String, Object> headers = call.argument("headers");
        flhvNetworkHelper.makeOCRCall(activity, endpoint, documentUri, parameters, headers, callback);
      } else if (call.method.equalsIgnoreCase("networkHelperMakeFaceMatchCall")) {
        APICompletionCallback callback = new APICompletionCallback() {
          @Override
          public void onResult(HVError hvError, HVResponse hvResponse) {
            ArrayList<Map<String, Object>> arr = flhvNetworkHelper.getArrayListFromOnResultNetworkHelper(hvError, hvResponse);
            Map<String, Object> map = new HashMap<>();
            map.put("errorObj", arr.get(0));
            map.put("resultObj", arr.get(1));
            result.success(map);
          }
        };
        String endpoint = call.argument("endpoint");
        String documentUri = call.argument("documentUri");
        String faceUri = call.argument("faceUri");
        Map<String, Object> parameters = call.argument("parameters");
        Map<String, Object> headers = call.argument("headers");
        flhvNetworkHelper.makeFaceMatchCall(activity, endpoint, documentUri, faceUri, parameters, headers, callback);
      } else {
        result.notImplemented();
      }
    }
  };
  MethodCallHandler hvQRScanCaptureMethodCallHandler = new MethodCallHandler() {
    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull final Result result) {
      if (call.method.equalsIgnoreCase("qrScanCaptureStart")) {
        QRScannerCompletionHandler callback = new QRScannerCompletionHandler() {
          @RequiresApi(api = Build.VERSION_CODES.KITKAT)
          @Override
          public void onResult(HVError hvError, JSONObject hvResponse) {
            ArrayList<Map<String, Object>> arr = flhvqrScanCapture.getALFromOnResultQRScan(hvError, hvResponse);
            Map<String, Object> map = new HashMap<>();
            map.put("errorObj", arr.get(0));
            map.put("resultObj", arr.get(1));
            result.success(map);
          }
        };
        flhvqrScanCapture.start(activity, callback);
      } else {
        result.notImplemented();
      }
    }
  };


  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    this.context = flutterPluginBinding.getApplicationContext();
    hyperSnapSDKChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hyperSnapSDK");
    hyperSnapParamsChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hyperSnapParams");
    hvDocsCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvDocsCapture");
    hvFaceCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvFaceCapture");
    hvNetworkHelperChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvNetworkHelper");
    hvQRScanCaptureChannel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hvQRScanCapture");

    hyperSnapSDKChannel.setMethodCallHandler(hyperSnapSDKMethodCallHandler);
    hyperSnapParamsChannel.setMethodCallHandler(hyperSnapParamsMethodCallHandler);
    hvDocsCaptureChannel.setMethodCallHandler(hvDocsCaptureMethodCallHandler);
    hvFaceCaptureChannel.setMethodCallHandler(hvFaceCaptureMethodCallHandler);
    hvNetworkHelperChannel.setMethodCallHandler(hvNetworkHelperMethodCallHandler);
    hvQRScanCaptureChannel.setMethodCallHandler(hvQRScanCaptureMethodCallHandler);
  }


  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
    hyperSnapSDKChannel.setMethodCallHandler(null);
    hyperSnapParamsChannel.setMethodCallHandler(null);
    hvDocsCaptureChannel.setMethodCallHandler(null);
    hvFaceCaptureChannel.setMethodCallHandler(null);
    hvNetworkHelperChannel.setMethodCallHandler(null);
    hvQRScanCaptureChannel.setMethodCallHandler(null);
  }

  @Override
  public void onAttachedToActivity(@NonNull ActivityPluginBinding binding) {
    this.activity = binding.getActivity();
  }

  @Override
  public void onDetachedFromActivityForConfigChanges() {

  }

  @Override
  public void onReattachedToActivityForConfigChanges(@NonNull ActivityPluginBinding binding) {

  }

  @Override
  public void onDetachedFromActivity() {

  }
}
