//
//  FLHVDocsCapture.swift
//


import Foundation
import HyperSnapSDK

public class FLHVDocsCapture{
    
    static var docConfig = HVDocConfig()
    static var docType: String?
    static var doc: HyperSnapParams.DocumentType?
    static var aspectRatio: Float?
    
    public static func getDocConfig() -> HVDocConfig {
        if (FLHVDocsCapture.docConfig == nil){
            FLHVDocsCapture.docConfig = HVDocConfig()
        }
        
        return FLHVDocsCapture.docConfig
    }
    
    public static func setOCRAPIDetails(ocrEndpoint: String, documentSideString: String, params: Dictionary<String, Any>, headers: Dictionary<String, Any>){
        
        let documentSideParams = documentSideString.components(separatedBy: ".")
        let documentSide = getDocumentSide(documentSide: documentSideParams[2])
        
        if (documentSide != nil){
            docConfig.setOCRAPIDetails(ocrEndpoint, documentSide: documentSide, params: params as! [String : AnyObject], headers: headers as! [String : String])
        }
    }
    
    
    public static func getDocumentSide(documentSide: String) -> HVDocConfig.DocumentSide{
        switch documentSide {
        case "FRONT", "front":
            return HVDocConfig.DocumentSide.front
        case "BACK", "back":
            return HVDocConfig.DocumentSide.back
        default:
            return HVDocConfig.DocumentSide.front
        }
    }
    
    
    public static func setAspectRatio(aspectRatioValue: Double){
        aspectRatio = Float(aspectRatioValue)
        FLHVDocsCapture.getDocConfig().setAspectRatio(aspectRatioValue)
    }
    
    public static func setShouldAddPadding(shouldSetPadding: Bool){
        FLHVDocsCapture.getDocConfig().setShouldAddPadding(shouldSetPadding)
    }
    
    
    public static func setShouldShowFlashIcon(shouldShowFlashIcon: Bool){
        FLHVDocsCapture.getDocConfig().setShouldShowFlashButton(shouldShowFlashIcon)
    }
    
    
    public static func setShouldShowReviewScreen(shouldShowReviewScreen: Bool){
        FLHVDocsCapture.getDocConfig().setShouldShowReviewPage(shouldShowReviewScreen)
    }
    
    
    public static func setDocCaptureSubText(subText: String){
        FLHVDocsCapture.getDocConfig().textConfig.setDocCaptureSubText(subText)
    }
    
    
    public static func setDocCaptureDescription(description: String){
        FLHVDocsCapture.getDocConfig().textConfig.setDocCaptureDescription(description)
    }
    
    
    public static func setShouldShowInstructionPage(shouldShowInstructionPage: Bool){
        FLHVDocsCapture.getDocConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
    }
    
    
    public static func getConfig() -> HVDocConfig {
        return FLHVDocsCapture.getDocConfig()
    }
    
    
    public static func setDocumentType(docTypeValue: String){
        self.docType = docTypeValue.components(separatedBy: ".")[1]
        if (self.docType == nil) {
            self.docType = "Card"
        }
        
        switch self.docType {
        case "a4":
            self.doc = HyperSnapParams.DocumentType.a4
        case "card":
            self.doc = HyperSnapParams.DocumentType.card
        case "other":
            self.doc = HyperSnapParams.DocumentType.other
        case "passport":
            self.doc = HyperSnapParams.DocumentType.passport
        default:
            self.doc = HyperSnapParams.DocumentType.card
        }
        
        FLHVDocsCapture.getDocConfig().setDocumentType(self.doc!)
    }
    
    
    public static func setDocCaptureTitle(titleText: String){
        FLHVDocsCapture.getDocConfig().textConfig.setDocCaptureTitle(titleText)
    }
    
    
    public static func setDocReviewTitle(docReviewTitle: String){
        FLHVDocsCapture.getDocConfig().textConfig.setDocReviewTitle(docReviewTitle)
    }
    
    public static func setDocReviewDescription(docReviewDescription: String){
        FLHVDocsCapture.getDocConfig().textConfig.setDocReviewDescription(docReviewDescription)
    }
    
    public static func start(context: UIViewController, callback: @escaping ((HVError?,HVResponse?,UIViewController)->Void)){

        HVDocsViewController.start(context, hvDocConfig: HVDocConfig(), completionHandler: callback)
    }
}
