//
//  FLHVFaceCapture.swift
//

import Foundation
import HyperSnapSDK

public class FLHVFaceCapture{
    
    static var faceConfig = HVFaceConfig()
    static var liveness: String?
    static var livenessMode: HyperSnapParams.LivenessMode?
    
    public static func getFaceConfig() -> HVFaceConfig {
        if (FLHVDocsCapture.docConfig == nil){
            FLHVDocsCapture.docConfig = HVDocConfig()
        }
        
        return FLHVFaceCapture.faceConfig
    }
    
    public static func setLivenessEndpoint(livenessEndpoint: String){
        FLHVFaceCapture.getFaceConfig().setLivenessEndpoint(livenessEndpoint)
    }
    
    public static func setShouldShowCameraSwitchButton(shouldShow: Bool){
        getFaceConfig().setShouldShowCameraSwitchButton(shouldShow)
    }
    
    public static func setShouldShowInstructionPage(shouldShowInstructionPage: Bool){
        getFaceConfig().setShouldShowInstructionsPage(shouldShowInstructionPage)
    }
    
    public static func setLivenessMode(livenessValue: String){
        liveness = livenessValue.components(separatedBy: ".")[1]
        
        switch liveness {
        case "none":
            livenessMode = HyperSnapParams.LivenessMode.none
        case "textureLiveness":
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        default:
            livenessMode = HyperSnapParams.LivenessMode.textureLiveness
        }
        
        getFaceConfig().setLivenessMode(livenessMode!)
    }
    
    public static func setFaceCaptureTitle(faceCaptureTitle: String){
        getFaceConfig().textConfig.setFaceCaptureTitle(faceCaptureTitle)
    }
    
    public static func setShouldUseBackCamera(shouldUseBackCamera: Bool){
        getFaceConfig().setShouldUseBackCamera(shouldUseBackCamera)
    }
    
    public static func setShouldAddPadding(shouldSetPadding: Bool){
        getFaceConfig().setShouldEnablePadding(shouldSetPadding)
    }
    
    public static func setPadding(leftPadding: Double, rightPadding: Double, topPadding: Double, bottomPadding: Double){
        getFaceConfig().setPadding(left: Float(leftPadding), right: Float(rightPadding), top: Float(topPadding), bottom: Float(bottomPadding))
        
    }
    
    public static func setLivenessAPIParameters(params: String){
        
        let jsonData = params.data(using: .utf8)!
        do{
            if let jsonObject = try JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as? [String : AnyObject]{
                getFaceConfig().setLivenessAPIParameters(jsonObject)
            }
        } catch let error as NSError {
            print(error)
        }
    }
    
    public static func setLivenessAPIHeaders(headers: String){
        let jsonData = headers.data(using: .utf8)!
        do{
            if let jsonObject = try JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as? [String : String]{
                getFaceConfig().setLivenessAPIHeaders(jsonObject)
            }
        } catch let error as NSError {
            print(error)
        }
    }
    
    
    public static func start(context: UIViewController, callback: @escaping ((HVError?,HVResponse?,UIViewController)->Void)) {
        
        HVFaceViewController.start(context, hvFaceConfig: HVFaceConfig(), completionHandler: callback)
    }
}
