import Flutter
import UIKit
import HyperSnapSDK


public class SwiftHypersnapsdkFlutterPlugin: NSObject, FlutterPlugin {
    
    var flHypersnapSDK: FLHyperSnapSDK?
    var flHyperSnapParams: FLHyperSnapParams?
    var flhvDocsCapture: FLHVDocsCapture?
    var flhvFaceCapture: FLHVFaceCapture?
    var flhvqrScanCapture: FLHVQRScanCapture?
    var flhvNetworkHelper: FLHVNetworkHelper?
    
  public static func register(with registrar: FlutterPluginRegistrar) {
    
    let hyperSnapSDKChannel = FlutterMethodChannel(name: "hyperSnapSDK", binaryMessenger: registrar.messenger())
    let hyperSnapParamsChannel = FlutterMethodChannel(name: "hyperSnapParams", binaryMessenger: registrar.messenger())
    let hvDocsCaptureChannel = FlutterMethodChannel(name: "hvDocsCapture", binaryMessenger: registrar.messenger())
    let hvFaceCaptureChannel = FlutterMethodChannel(name: "hvFaceCapture", binaryMessenger: registrar.messenger())
    let hvNetworkHelperChannel = FlutterMethodChannel(name: "hvNetworkHelper", binaryMessenger: registrar.messenger())
    let hvQRScanCaptureChannel = FlutterMethodChannel(name: "hvQRScanCapture", binaryMessenger: registrar.messenger())
    
    
    
    // Handler for HyperSnapSDK
    hyperSnapSDKChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: FlutterResult) -> Void in
        
        guard let args = call.arguments as? Dictionary<String, Any> else {
            //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            return
        }
        
        if (call.method.caseInsensitiveCompare("initialize") == .orderedSame){
            
            let appID = args["appID"] as? String
            let appKey = args["appKey"] as? String
            let region = args["region"] as? String
            
            FLHyperSnapSDK.initialize(appId: appID!, appName: appKey!, regionValue: region!)
            result(String("Initialised"))
            
        } else if (call.method.caseInsensitiveCompare("startUserSession") == .orderedSame){
            let userID = args["userID"] as? String
            FLHyperSnapSDK.startUserSession(userID: userID!)
        } else if (call.method.caseInsensitiveCompare("endUserSession") == .orderedSame){
            FLHyperSnapSDK.endUserSession()
        } else {
            result(FlutterMethodNotImplemented)
        }
    })
    
    
    
    // Handler for HyperSnapParams
    hyperSnapParamsChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: FlutterResult) -> Void in
        
        if (call.method.caseInsensitiveCompare("getValidParams") == .orderedSame){
            result(FLHyperSnapParams.getConstants())
        }
    })
    
    
    
    
    // Handler for FLHVDocsCapture
    hvDocsCaptureChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
        
        if (call.method.caseInsensitiveCompare("docsCaptureSetOCRAPIDetails") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let ocrEndpoint = args["ocrEndpoint"] as! String
            let documentSideString = args["documentSideString"] as! String
            let params = args["params"] as! Dictionary<String, Any>
            let headers = args["headers"] as! Dictionary<String, Any>
            
            FLHVDocsCapture.setOCRAPIDetails(ocrEndpoint: ocrEndpoint, documentSideString: documentSideString, params: params, headers: headers)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetAspectRatio") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let aspectRatioValue = args["aspectRatioValue"] as! Double
            FLHVDocsCapture.setAspectRatio(aspectRatioValue: aspectRatioValue)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetShouldAddPadding") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldSetPadding = args["shouldSetPadding"] as! Bool
            FLHVDocsCapture.setShouldAddPadding(shouldSetPadding: shouldSetPadding)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetShouldShowFlashIcon") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldShowFlashIcon = args["shouldShowFlashIcon"] as! Bool
            FLHVDocsCapture.setShouldShowFlashIcon(shouldShowFlashIcon: shouldShowFlashIcon)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetShouldShowReviewScreen") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldShowReviewScreen = args["shouldShowReviewScreen"] as! Bool
            FLHVDocsCapture.setShouldShowReviewScreen(shouldShowReviewScreen: shouldShowReviewScreen)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocCaptureSubText") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let subText = args["subText"] as! String
            FLHVDocsCapture.setDocCaptureSubText(subText: subText)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocCaptureDescription") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let description = args["description"] as! String
            FLHVDocsCapture.setDocCaptureDescription(description: description)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetShouldShowInstructionPage") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldShowInstructionPage = args["shouldShowInstructionPage"] as! Bool
            FLHVDocsCapture.setShouldShowInstructionPage(shouldShowInstructionPage: shouldShowInstructionPage)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureGetConfig") == .orderedSame){
            result(FLHVDocsCapture.getDocConfig())
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocumentType") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let docTypeValue = args["docTypeValue"] as! String
            FLHVDocsCapture.setDocumentType(docTypeValue: docTypeValue)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocCaptureTitle") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let titleText = args["titleText"] as! String
            FLHVDocsCapture.setDocCaptureTitle(titleText: titleText)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocReviewTitle") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let docReviewTitle = args["docReviewTitle"] as! String
            FLHVDocsCapture.setDocReviewTitle(docReviewTitle: docReviewTitle)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureSetDocReviewDescription") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let docReviewDescription = args["docReviewDescription"] as! String
            FLHVDocsCapture.setDocReviewDescription(docReviewDescription: docReviewDescription)
        }
        else if (call.method.caseInsensitiveCompare("docsCaptureStart") == .orderedSame){
            let completionHandler: (_ error: HVError?,_ response: HVResponse?,_ viewController: UIViewController)-> Void = {error, hvResult, vcNew in
                
                var errorObj = [String : Any]()
                var resultObj = [String : String]()

                if(error != nil){/*Handle error*/
                    //Error Code: error.errorCode, Error Message: error.errorMessage
                    errorObj["errorCode"] = error?.errorCode
                    errorObj["errorMessage"] = error?.errorMessage
             
                  } else {
                    if let result = hvResult {
                        let response = result.apiResult
                        
                        do{
//                            let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
//                            let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                            
                            let jsonString = response?.description
                            
                            resultObj["apiResult"] = jsonString
                            
                            if let imageUri = result.imageUri {
                                if !imageUri.isEmpty {
                                    resultObj["imageUri"] = imageUri
                                }
                            }
                            
                            if let fullImageUri = result.fullImageUri {
                                if !fullImageUri.isEmpty{
                                    resultObj["fullImageUri"] = fullImageUri
                                }
                            }
                            
                            if let retakeMessage = result.retakeMessage {
                                if !retakeMessage.isEmpty {
                                    resultObj["retakeMessage"] = retakeMessage
                                }
                            }
                            
                            if let action = result.action {
                                if !action.isEmpty {
                                    resultObj["action"] = action
                                }
                            }
                            
                            if let headers = result.apiHeaders {
//                                let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
//                                let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                
                                let headersString = headers.description
                                
                                resultObj["apiHeaders"] = headersString
                                
                            }
                        }
                        catch let myJSONError {
                            print(myJSONError)
                        }
                    }
                    
                    
                    
                 }
                
                let map = [
                    "errorObj": errorObj,
                    "resultObj": resultObj,
                ]
                
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController{
                FLHVDocsCapture.start(context: topController, callback: completionHandler)
            }
            
        }
        else{
            result(FlutterMethodNotImplemented)
        }
    })
    
    
    
    // Handler for FLHVFaceCapture
    hvFaceCaptureChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in

        if (call.method.caseInsensitiveCompare("faceCaptureStart") == .orderedSame){
            let completionHandler: (_ error: HVError?,_ response: HVResponse?,_ viewController: UIViewController)-> Void = {error, hvResult, vcNew in
                
                var errorObj = [String : Any]()
                var resultObj = [String : String]()

                if(error != nil){/*Handle error*/
                    //Error Code: error.errorCode, Error Message: error.errorMessage
                    errorObj["errorCode"] = error?.errorCode
                    errorObj["errorMessage"] = error?.errorMessage
             
                  } else {
                    if let result = hvResult {
                        let response = result.apiResult
                        
                        do{
//                            let jsonData = try JSONSerialization.data(withJSONObject: response, options: JSONSerialization.WritingOptions.prettyPrinted)
//                            let jsonString = String(data: jsonData, encoding: String.Encoding.utf8)!
                            
                            let jsonString = response?.description
                            resultObj["apiResult"] = jsonString
                            
                            if let imageUri = result.imageUri {
                                if !imageUri.isEmpty {
                                    resultObj["imageUri"] = imageUri
                                }
                            }
                            
                            if let fullImageUri = result.fullImageUri {
                                if !fullImageUri.isEmpty{
                                    resultObj["fullImageUri"] = fullImageUri
                                }
                            }
                            
                            if let retakeMessage = result.retakeMessage {
                                if !retakeMessage.isEmpty {
                                    resultObj["retakeMessage"] = retakeMessage
                                }
                            }
                            
                            if let action = result.action {
                                if !action.isEmpty {
                                    resultObj["action"] = action
                                }
                            }
                            
                            if let headers = result.apiHeaders {
//                                let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
//                                let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                
                                let headersString = headers.description
                                
                                resultObj["apiHeaders"] = headersString
                                
                            }
                        }
                        catch let myJSONError {
                            print(myJSONError)
                        }
                    }
                 
                 }
                
                let map = [
                    "errorObj": errorObj,
                    "resultObj": resultObj,
                ]
                
                result(map)
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController{
                FLHVFaceCapture.start(context: topController, callback: completionHandler)
            }
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetLivenessEndpoint") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let livenessEndpoint = args["livenessEndpoint"] as! String
            FLHVFaceCapture.setLivenessEndpoint(livenessEndpoint: livenessEndpoint)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetShouldShowCameraSwitchButton") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldShow = args["shouldShow"] as! Bool
            FLHVFaceCapture.setShouldShowCameraSwitchButton(shouldShow: shouldShow)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetShouldShowInstructionPage") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldShowInstructionPage = args["shouldShowInstructionPage"] as! Bool
            FLHVFaceCapture.setShouldShowInstructionPage(shouldShowInstructionPage: shouldShowInstructionPage)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetLivenessMode") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let livenessValue = args["livenessValue"] as! String
            FLHVFaceCapture.setLivenessMode(livenessValue: livenessValue)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetFaceCaptureTitle") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let faceCaptureTitle = args["faceCaptureTitle"] as! String
            FLHVFaceCapture.setFaceCaptureTitle(faceCaptureTitle: faceCaptureTitle)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetShouldUseBackCamera") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldUseBackCamera = args["shouldUseBackCamera"] as! Bool
            FLHVFaceCapture.setShouldUseBackCamera(shouldUseBackCamera: shouldUseBackCamera)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetShouldAddPadding") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let shouldSetPadding = args["shouldSetPadding"] as! Bool
            FLHVFaceCapture.setShouldAddPadding(shouldSetPadding: shouldSetPadding)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetPadding") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let leftPadding = args["leftPadding"] as! Double
            let rightPadding = args["rightPadding"] as! Double
            let topPadding = args["topPadding"] as! Double
            let bottomPadding = args["bottomPadding"] as! Double
            FLHVFaceCapture.setPadding(leftPadding: leftPadding, rightPadding: rightPadding, topPadding: topPadding, bottomPadding: bottomPadding)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetLivenessAPIParameters") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let params = args["params"] as! String
            FLHVFaceCapture.setLivenessAPIParameters(params: params)
        }
        else if (call.method.caseInsensitiveCompare("faceCaptureSetLivenessAPIHeaders") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                return result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
            }
            
            let headers = args["headers"] as! String
            FLHVFaceCapture.setLivenessAPIHeaders(headers: headers)
        }
        else{
            result(FlutterMethodNotImplemented)
        }
    })
    
    
    
    // Handler for FLHVNetworkHelper
    hvNetworkHelperChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
        
        if (call.method.caseInsensitiveCompare("networkHelperMakeOCRCall") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                return
            }
            let endpoint = args["endpoint"] as? String
            let documentUri = args["documentUri"] as? String
            let parameters = args["parameters"] as? NSDictionary
            let headers = args["headers"] as? NSDictionary
            let completionHandler: (_ error: HVError?,_ response: HVResponse?)-> Void = {error, hvResult in
                
                var errorObj = [String : Any]()
                var resultObj = [String : String]()

                if(error != nil){/*Handle error*/
                    //Error Code: error.errorCode, Error Message: error.errorMessage
                    errorObj["errorCode"] = error?.errorCode
                    errorObj["errorMessage"] = error?.errorMessage
             
                  } else {
                    if let result = hvResult {
                        let response = result.apiResult
                        
                        do{
                            let jsonString = response?.description
                            
                            resultObj["apiResult"] = jsonString
                            
                            if let imageUri = result.imageUri {
                                if !imageUri.isEmpty {
                                    resultObj["imageUri"] = imageUri
                                }
                            }
                            
                            if let fullImageUri = result.fullImageUri {
                                if !fullImageUri.isEmpty{
                                    resultObj["fullImageUri"] = fullImageUri
                                }
                            }
                            
                            if let retakeMessage = result.retakeMessage {
                                if !retakeMessage.isEmpty {
                                    resultObj["retakeMessage"] = retakeMessage
                                }
                            }
                            
                            if let action = result.action {
                                if !action.isEmpty {
                                    resultObj["action"] = action
                                }
                            }
                            
                            if let headers = result.apiHeaders {
//                                let headersData = try JSONSerialization.data(withJSONObject: headers, options: JSONSerialization.WritingOptions.prettyPrinted)
//                                let headersString = String(data: headersData, encoding: String.Encoding.utf8)
                                
                                let headersString = headers.description
                                
                                resultObj["apiHeaders"] = headersString
                                
                            }
                        }
                        catch let myJSONError {
                            print(myJSONError)
                        }
                    }
                 
                 }
                
                let map = [
                    "errorObj": errorObj,
                    "resultObj": resultObj,
                ]
                
                result(map)
            }
            
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController{
                FLHVNetworkHelper.makeOCRCall(context: topController, endpoint: endpoint!, documentUri: documentUri!, parameters: parameters!, headers: headers!, callback: completionHandler)
            }
            
            
            
        }
        else if (call.method.caseInsensitiveCompare("networkHelperMakeFaceMatchCall") == .orderedSame){
            guard let args = call.arguments as? Dictionary<String, Any> else {
                //result(String("iOS could not recognize flutter arguments in method: (sendParams)"))
                return
            }
            let endpoint = args["endpoint"] as? String
            let documentUri = args["documentUri"] as? String
            let faceUri = args["faceUri"] as? String
            let parameters = args["parameters"] as? NSDictionary
            let headers = args["headers"] as? NSDictionary
            let completionHandler: (_ error: HVError?,_ response: HVResponse?)-> Void = {error, hvResult in
                
                var errorObj = [String : Any]()
                var resultObj = [String : String]()

                if(error != nil){/*Handle error*/
                    //Error Code: error.errorCode, Error Message: error.errorMessage
                    errorObj["errorCode"] = error?.errorCode
                    errorObj["errorMessage"] = error?.errorMessage
             
                  } else {
                    if let result = hvResult {
                        let response = result.apiResult
                        
                        do{
                            let jsonString = response?.description
                            
                            resultObj["apiResult"] = jsonString
                            
                            if let imageUri = result.imageUri {
                                if !imageUri.isEmpty {
                                    resultObj["imageUri"] = imageUri
                                }
                            }
                            
                            if let fullImageUri = result.fullImageUri {
                                if !fullImageUri.isEmpty{
                                    resultObj["fullImageUri"] = fullImageUri
                                }
                            }
                            
                            if let retakeMessage = result.retakeMessage {
                                if !retakeMessage.isEmpty {
                                    resultObj["retakeMessage"] = retakeMessage
                                }
                            }
                            
                            if let action = result.action {
                                if !action.isEmpty {
                                    resultObj["action"] = action
                                }
                            }
                            
                            if let headers = result.apiHeaders {
                                let headersString = headers.description
                                
                                resultObj["apiHeaders"] = headersString
                                
                            }
                        }
                        catch let myJSONError {
                            print(myJSONError)
                        }
                    }
                 
                 }
                
                let map = [
                    "errorObj": errorObj,
                    "resultObj": resultObj,
                ]
                
                result(map)
            }
            
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController{
                FLHVNetworkHelper.makeFaceMatchCall(context: topController, endpoint: endpoint!, faceUri: faceUri!, documentUri: documentUri!, parameters: parameters!, headers: headers!, callback: completionHandler)
            }
            
        }
        else{
            result(FlutterMethodNotImplemented)
        }
    })
    
    hvQRScanCaptureChannel.setMethodCallHandler({
        (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
        
        if(call.method.caseInsensitiveCompare("qrScanCaptureStart") == .orderedSame){
            
            let completionHandler: (_ error: HVError?, _ result: NSDictionary?) -> Void = { error, result in
                // What to do, What not to do
            }
            
            if let topController = UIApplication.shared.keyWindow?.rootViewController{
                FLHVQRScanCapture.start(context: topController, callback: completionHandler)
            }
        } else{
            result(FlutterMethodNotImplemented)
        }
        
        
    })
    
//    let instance = SwiftHvSdkWrapperPlugin()
//    registrar.addMethodCallDelegate(instance, channel: channel)
  }
    

//    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
//       if (call.method == "getPlatformVersion") {
//               result("iOS " + UIDevice.current.systemVersion)
//           }
//           else if (call.method == "showAlertDialog") {
//               DispatchQueue.main.async {
//                   let alert = UIAlertController(title: "Alert", message: "Hi, My name is flutter", preferredStyle: .alert);
//                alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil))
//                   UIApplication.shared.keyWindow?.rootViewController?.present(alert, animated: true, completion: nil);
//               }
//           }
//     }
}
