import 'dart:async';
import 'package:flutter/services.dart';

class HyperSnapSDKFlutter {
  static const MethodChannel _hyperSnapSDKChannel =
  const MethodChannel('hyperSnapSDK');
  static const MethodChannel _hvQRScanCaptureChannel =
  const MethodChannel('hvQRScanCapture');
  static const MethodChannel _hyperSnapParamsChannel =
  const MethodChannel('hyperSnapParams');
  static const MethodChannel _hvDocsCatpureChannel =
  const MethodChannel('hvDocsCapture');
  static const MethodChannel _hvFaceCaptureChannel =
  const MethodChannel('hvFaceCapture');
  static const MethodChannel _hvNetworkHelperChannel =
  const MethodChannel('hvNetworkHelper');

  /// The following functions (initialize(...), startUserSession(userID), endUserSession()) are related to
  /// hyperSnapSDK channel and its methods

  static Future<String> initialize(
      String appID, String appKey, String region) async {
    final String init = await _hyperSnapSDKChannel.invokeMethod(
        'initialize', {"appID": appID, "appKey": appKey, "region": region});
    return init;
  }

  static startUserSession(String userID) async {
    await _hyperSnapSDKChannel
        .invokeMethod('startUserSession', {"userID": userID});
  }

  static endUserSession() async {
    await _hyperSnapSDKChannel.invokeMethod('endUserSession');
  }

  /// The following functions () are related to hyperSnapParams channel and its methods

  static Future<Map> getValidParams() async {
    final Map map =
    await _hyperSnapParamsChannel.invokeMethod('getValidParams');
    return map;
  }

  /// The following functions () are related to hvDocsCapture channel and its methods

  static docsCaptureSetOCRAPIDetails(String ocrEndpoint,
      String documentSideString, Map params, Map headers) async {
    await _hvDocsCatpureChannel.invokeMethod('docsCaptureSetOCRAPIDetails', {
      "ocrEndpoint": ocrEndpoint,
      "documentSideString": documentSideString,
      "params": params,
      "headers": headers
    });
  }

  static docsCaptureSetCustomUIStrings(String customStrings) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetCustomUIStrings', {"customStrings": customStrings});
  }

  static docsCaptureSetAspectRatio(double aspectRatioValue) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetAspectRatio', {"aspectRatioValue": aspectRatioValue});
  }

  static docsCaptureSetShouldAddPadding(bool shouldSetPadding) async {
    await _hvDocsCatpureChannel.invokeMethod('docsCaptureSetShouldAddPadding',
        {"shouldSetPadding": shouldSetPadding});
  }

  static docsCaptureSetPadding(double padding) async {
    await _hvDocsCatpureChannel
        .invokeMethod('docsCaptureSetPadding', {"padding": padding});
  }

  static docsCaptureSetShouldShowFlashIcon(bool shouldShowFlashIcon) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetShouldShowFlashIcon',
        {"shouldShowFlashIcon": shouldShowFlashIcon});
  }

  static docsCaptureSetShouldShowReviewScreen(
      bool shouldShowReviewScreen) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetShouldShowReviewScreen',
        {"shouldShowReviewScreen": shouldShowReviewScreen});
  }

  static docsCaptureSetDocCaptureSubText(String subText) async {
    await _hvDocsCatpureChannel
        .invokeMethod('docsCaptureSetDocCaptureSubText', {"subText": subText});
  }

  static docsCaptureSetDocCaptureDescription(String description) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetDocCaptureDescription', {"description": description});
  }

  static docsCaptureSetShouldShowInstructionPage(
      bool shouldShowInstructionPage) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetShouldShowInstructionPage',
        {"shouldShowInstructionPage": shouldShowInstructionPage});
  }

  static Future<Object> docsCaptureGetConfig() async {
    await _hvDocsCatpureChannel.invokeMethod('docsCaptureGetConfig');
    return new Object();
  }

  static docsCaptureSetDocumentType(String docTypeValue) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetDocumentType', {"docTypeValue": docTypeValue});
  }

  static docsCaptureSetDocCaptureTitle(String titleText) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetDocCaptureTitle', {"titleText": titleText});
  }

  static docsCaptureSetDocReviewTitle(String docReviewTitle) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetDocReviewTitle', {"docReviewTitle": docReviewTitle});
  }

  static docsCaptureSetDocReviewDescription(String docReviewDescription) async {
    await _hvDocsCatpureChannel.invokeMethod(
        'docsCaptureSetDocReviewDescription',
        {"docReviewDescription": docReviewDescription});
  }

  static Future<Map> docsCaptureStart() async {
    final Map map =
    await _hvDocsCatpureChannel.invokeMethod('docsCaptureStart');
    return map;
  }

  /// The following functions () are related to hvFaceCapture channel and its methods

  static Future<Map> faceCaptureStart() async {
    final Map map =
    await _hvFaceCaptureChannel.invokeMethod('faceCaptureStart');
    return map;
  }

  static faceCaptureSetLivenessEndpoint(String livenessEndpoint) async {
    await _hvFaceCaptureChannel.invokeMethod('faceCaptureSetLivenessEndpoint',
        {"livenessEndpoint": livenessEndpoint});
  }

  static faceCaptureSetShouldReturnFullImageUrl(
      bool shouldReturnFullImageUrl) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetShouldReturnFullImageUrl',
        {"faceCaptureSetShouldReturnFullImageUrl": shouldReturnFullImageUrl});
  }

  static faceCaptureSetClientID(String clientID) async {
    await _hvFaceCaptureChannel
        .invokeMethod('faceCaptureSetClientID', {"clientID": clientID});
  }

  static faceCaptureSetCustomUIStrings(String customStrings) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetCustomUIStrings', {"customStrings": customStrings});
  }

  static faceCaptureSetShouldShowCameraSwitchButton(bool shouldShow) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetShouldShowCameraSwitchButton',
        {"shouldShow": shouldShow});
  }

  static faceCaptureSetShouldShowInstructionPage(
      bool shouldShowInstructionPage) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetShouldShowInstructionPage',
        {"shouldShowInstructionPage": shouldShowInstructionPage});
  }

  static faceCaptureSetLivenessMode(String livenessValue) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetLivenessMode', {"livenessValue": livenessValue});
  }

  static faceCaptureSetFaceCaptureTitle(String faceCaptureTitle) async {
    await _hvFaceCaptureChannel.invokeMethod('faceCaptureSetFaceCaptureTitle',
        {"faceCaptureTitle": faceCaptureTitle});
  }

  static faceCaptureSetShouldUseBackCamera(bool shouldUseBackCamera) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetShouldUseBackCamera',
        {"shouldUseBackCamera": shouldUseBackCamera});
  }

  static faceCaptureSetShouldEnableDataLogging(bool dataLogging) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetShouldEnableDataLogging', {"dataLogging": dataLogging});
  }

  static faceCaptureSetShouldAddPadding(bool shouldSetPadding) async {
    await _hvFaceCaptureChannel.invokeMethod('faceCaptureSetShouldAddPadding',
        {"shouldSetPadding": shouldSetPadding});
  }

  static faceCaptureSetPadding(double leftPadding, double rightPadding,
      double topPadding, double bottomPadding) async {
    await _hvFaceCaptureChannel.invokeMethod('faceCaptureSetPadding', {
      "leftPadding": leftPadding,
      "rightPadding": rightPadding,
      "topPadding": topPadding,
      "bottomPadding": bottomPadding
    });
  }

  static faceCaptureSetLivenessAPIParameters(String params) async {
    await _hvFaceCaptureChannel.invokeMethod(
        'faceCaptureSetLivenessAPIParameters', {"params": params});
  }

  static faceCaptureSetLivenessAPIHeaders(String headers) async {
    await _hvFaceCaptureChannel
        .invokeMethod('faceCaptureSetLivenessAPIHeaders', {"headers": headers});
  }

  /// The following functions () are related to hvNetworkHelper channel and its methods

  static Future<Map> networkHelperMakeOCRCall(
      String endpoint, String documentUri, Map parameters, Map headers) async {
    final Map map =
    await _hvNetworkHelperChannel.invokeMethod('networkHelperMakeOCRCall', {
      "endpoint": endpoint,
      "documentUri": documentUri,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }

  static Future<Map> networkHelperMakeFaceMatchCall(String endpoint,
      String faceUri, String documentUri, Map parameters, Map headers) async {
    final Map map = await _hvNetworkHelperChannel
        .invokeMethod('networkHelperMakeFaceMatchCall', {
      "endpoint": endpoint,
      "faceUri": faceUri,
      "documentUri": documentUri,
      "parameters": parameters,
      "headers": headers
    });
    return map;
  }

  /// The following functions () are related to hvQRScanCapture channel and its methods

  static Future<Map> qrScanCaptureStart() async {
    final Map map =
    await _hvQRScanCaptureChannel.invokeMethod('qrScanCaptureStart');
    return map;
  }
}